<?php
/**
 * Add Shortcode
 *
 * @category   Plugin
 * @package    WordPress
 * @subpackage Shinobi Reviews
 * @author     Shinobi Works <support@shinobiworks.com>
 * @license    https://www.gnu.org/licenses/gpl-3.0.html/ GPL v3 or later
 * @link       https://shinobiworks.com/
 * @since      1.0.0
 */

namespace Shinobi_Reviews\App;

use Shinobi_Reviews\App\Membership\Membership;
use Shinobi_Reviews\App\Module\Recaptcha;
use Shinobi_Works\WP\DB;

/**
 * Add shortcode class
 */
class AddShortcode {

	public function __construct() {
		add_shortcode( SHINOBI_REVIEWS, [ $this, 'shortcode' ] );
	}

	/**
	 * Shortcode Setting
	 *
	 * @param array $raw_atts raw attribute of shortcode.
	 * @link https://codex.wordpress.org/Shortcode_API/
	 * @return string
	 */
	public function shortcode( $raw_atts ) {
		$atts = shortcode_atts(
			[
				'id'            => null,
				'is_membership' => false,
			],
			$raw_atts,
			SHINOBI_REVIEWS
		);

		$form_id = $atts['id'];

		// get the all form data of Shinobi Reviews
		$all_form_data = DB::get_option( 'shinobi_reviews_form_data' );

		if ( ! $all_form_data ) {
			return '<p>' . __( 'Not found the shinobi-reviews form.', 'shinobi-reviews' ) . '</p>';
		}

		if ( ! isset( $all_form_data[ $form_id ] ) ) {
			return '<p>' . __( 'Not found the shinobi-reviews form.', 'shinobi-reviews' ) . '</p>';
		}

		// set the form data
		$form_data = $all_form_data[ $form_id ];

		// Since ready to enqueue plugin files, let's start!
		wp_enqueue_script( SHINOBI_REVIEWS );
		wp_enqueue_script( 'shinobi-reviews/google-recaptcha' );

		// get all reviews
		$review_data = $this->process_review_data( $form_id );

		wp_localize_script(
			SHINOBI_REVIEWS,
			'shinobiReviews',
			[
				'reviewData'         => $review_data,
				'formId'             => (int) $form_id,
				'locale'             => get_locale(),
				'signIn'             => [
					'token'  => DB::get_option( 'shinobiReviewsAuthToken' ) ? 'O9WJ9ruwbCaTQPy+fYyb5y0ywa7wMDt1mLKLpyysPyaDGILZhTgyWiceGKI5psUO' : '',
					'action' => 'sign_in_by_token',
					'nonce'  => wp_create_nonce( 'sign_in_by_token' ),
				],
				'initialReviewCount' => DB::get_option( 'initialReviewCount' ),
				'formActions'        => Membership::form_actions() + InsertReview::form_actions(),
				'recaptcha'          => Recaptcha::form_actions(),
			]
		);

		// Generate structured data
		if ( $review_data ) {
			$review_count = count( $review_data );

			$form_data += [
				'aggregateRating' => [
					'@type'       => 'AggregateRating',
					'ratingValue' => round( array_sum( array_column( $review_data, 'rating' ) ) / $review_count, 1 ),
					'ratingCount' => $review_count,
					'reviewCount' => $review_count,
				],
			];
		}
		$structured_data = '<script type="application/ld+json">' . wp_json_encode( $form_data ) . '</script>';

		return '<div id="shinobi-reviews"></div>' . $structured_data;
	}

	/**
	 * Process the review data for JS
	 *
	 * @return array|null
	 */
	private function process_review_data( $form_id ) {
		$all_review_data = DB::get_results( SHINOBI_REVIEWS_LIST_TABLE );

		if ( ! $all_review_data ) {
			return null;
		}

		$filtered_review_data = [];

		foreach ( $all_review_data as $i => $data ) {
			if ( $data->review_form_id === $form_id ) {
				$filtered_review_data[] = $data;
			}
		}

		if ( ! $filtered_review_data ) {
			return null;
		}

		$new_review_data = [];

		foreach ( $filtered_review_data as $_i => $review_data ) {
			if ( '0' === $review_data->review_approved ) {
				continue;
			}

			$id        = "_$review_data->ID";
			$user_data = DB::get_row_by_id( SHINOBI_MEMBERSHIP_TABLE, $review_data->user_id );

			if ( $user_data && '0' === $user_data->user_status ) {
				continue;
			}

			$reviewer = $user_data ? $user_data->user_nicename : $review_data->review_author;

			$new_review_data[ $id ] = [
				'content'  => $review_data->review_content,
				'rating'   => (int) $review_data->review_rating,
				'reviewer' => $reviewer ? $reviewer : DB::get_option( 'defaultReviewerName', __( 'Unregistered user', 'shinobi-reviews' ) ),
				'isMember' => $user_data ? true : false,
				'date'     => wp_date( 'Y/m/d', strtotime( $review_data->review_date ) ),
			];

			$attachment_ids = json_decode( $review_data->review_attachment_ids, true );
			if ( $attachment_ids && is_array( $attachment_ids ) ) {
				$new_review_data[ $id ]['images'] = array_map(
					function( $attachment_id ) {
						return wp_get_attachment_image_src( $attachment_id, 'large' )[0];
					},
					$attachment_ids
				);
			}
		}

		return $new_review_data ? array_reverse( $new_review_data, true ) : null;
	}

}
